@@ -16,6 +16,7 @@ import gc
 import shutil
 from itertools import cycle
 from threading import Thread
+from threading import Event
 from threading import Lock
 
 # Avoid Not-Find-Display problem
@@ -51,6 +52,8 @@ class evolution:
 
     # Content
     pop = []
+    threadPool = []
+    teminateEvent = Event()
     generation = 0
     presentation = pptx.Presentation()
 
@@ -106,14 +109,9 @@ class evolution:
         self.combine_positive_epistasis = combine_positive_epistasis
         self.popsize = popsize
         self.mutop = mutop.split(',')
-        self.global_seed = global_seed
-
-        try:
-            with open('editmap.pickle', 'rb') as editFitMapFile:
-                self.editFitMap = pickle.load(editFitMapFile)
-                print(f'[Initializing GEVO] Previous EditFitMap found. {len(self.editFitMap)} entries loaded')
-        except FileNotFoundError:
-            pass
+        self.rng = {}
+        if global_seed is not None:
+            random.seed(global_seed)
 
         try:
             with open(llvm_src_filename, 'r') as f:
@@ -235,13 +233,24 @@ class evolution:
         pathlib.Path('stage').mkdir(exist_ok=True)
         if self.generation == 0:
             stageFileName = "stage/startedits.json"
+            rngFileName = "stage/startrng.pickle"
         else:
             stageFileName = "stage/" + str(self.generation) + ".json"
+            rngFileName = "stage/" + str(self.generation) + "_rng.pickle"
 
         with open(stageFileName, 'w') as fp:
             stage = [{'edits': ind.edits, 'fitness': ind.fitness.values} for ind in self.pop]
             json.dump(stage, fp, indent=2)
 
+        rng_state = {}
+        rng_state['global'] = random.getstate()
+        rng_state['cx'] = self.rng['cx'].getstate()
+        rng_state['mut'] = self.rng['mut'].getstate()
+        rng_state['cx_local'] = [ rng.getstate() for rng in self.rng['cx_local']]
+        rng_state['mut_local'] = [ rng.getstate() for rng in self.rng['mut_local']]
+        with open(rngFileName, 'wb') as fp:
+            pickle.dump(rng_state, fp)
+
     def mutLog(self):
         print("gen {}".format(self.generation), file=self.mutLogFile)
         print("Individual mutation opeartion statistic", file=self.mutLogFile)
@@ -425,6 +434,9 @@ class evolution:
         trial = 0
         operations = self.mutop
         while trial < individual.lineSize*2:
+            if self.teminateEvent.is_set():
+                return individual,
+
             line1 = rng.randint(1, individual.lineSize)
             line2 = rng.randint(1, individual.lineSize)
             while line1 == line2:
@@ -448,6 +460,8 @@ class evolution:
                 continue
 
             with lock:
+                if self.teminateEvent.is_set():
+                    return individual,
                 trial = trial + 1
                 fit = self.evaluate(test_ind, 'mut')
 
@@ -489,7 +503,9 @@ class evolution:
             return ind1, ind2 
 
         while trial < len(ind1.edits) + len(ind2.edits):
-            
+            if self.teminateEvent.is_set():
+                return ind1, ind2
+
             rng.shuffle(symmetricEdits)
             point = rng.randint(1, len(symmetricEdits)-1)
             cmd1 = intersectionEdits + symmetricEdits[:point]
@@ -503,6 +519,8 @@ class evolution:
                 continue
 
             with lock:
+                if self.teminateEvent.is_set():
+                    return ind1, ind2
                 fit1 = self.evaluate(child1, 'cx')
                 fit2 = self.evaluate(child2, 'cx')
 
@@ -569,6 +587,8 @@ class evolution:
             proc.wait()
             self.mutStats['infinite'] = self.mutStats['infinite'] + 1
             return None, None
+        except KeyboardInterrupt:
+            return None, None
 
         program_output = stdout.decode()
         cmpResult, err = self.resultCompare(program_output, testcase)
@@ -674,17 +694,14 @@ class evolution:
         return avg_fitness, max_err
 
     def evolve(self, resumeGen):
-        threadPool = []
-        if self.global_seed is not None:
-            random.seed(self.global_seed)
-        mut_rng = random.Random(random.getrandbits(16))
-        cx_rng = random.Random(random.getrandbits(16))
+        self.rng['mut'] = random.Random(random.getrandbits(16))
+        self.rng['cx'] = random.Random(random.getrandbits(16))
 
         if resumeGen == -1:
             # PopSize must be a multiple by 4 for SelTournamentDOD to function properly
             popSize = self.popsize
-            self.mut_local_rng = [ random.Random(random.getrandbits(16)) for i in range(popSize) ]
-            self.cx_local_rng = [ random.Random(random.getrandbits(16)) for i in range(popSize) ]
+            self.rng['mut_local'] = [ random.Random(random.getrandbits(16)) for i in range(popSize) ]
+            self.rng['cx_local'] = [ random.Random(random.getrandbits(16)) for i in range(popSize) ]
             print("Initialize the population. Size {}".format(popSize))
             self.pop = self.toolbox.population(n=popSize)
 
@@ -701,9 +718,9 @@ class evolution:
                         _ind1 = creator.Individual(self.initSrcEnc, self.mgpu)
                         _ind2 = creator.Individual(self.initSrcEnc, self.mgpu)
                         _ind3 = creator.Individual(self.initSrcEnc, self.mgpu)
-                        self.toolbox.mutate(_ind1, mut_rng)
-                        self.toolbox.mutate(_ind2, mut_rng)
-                        self.toolbox.mutate(_ind3, mut_rng)
+                        self.toolbox.mutate(_ind1, self.rng['mut'])
+                        self.toolbox.mutate(_ind2, self.rng['mut'])
+                        self.toolbox.mutate(_ind3, self.rng['mut'])
                         _ind = creator.Individual(self.initSrcEnc, self.mgpu, _ind1.edits + _ind2.edits + _ind3.edits)
                         _fit = self.evaluate(_ind)
                     ind.copy_from(_ind)
@@ -713,8 +730,10 @@ class evolution:
         else:
             if resumeGen == 0:
                 stageFileName = "stage/startedits.json"
+                rngFileName   = "stage/startrng.pickle"
             else:
                 stageFileName = "stage/" + str(resumeGen) + ".json"
+                rngFileName   = "stage/" + str(resumeGen) + "_rng.pickle"
 
             try:
                 stage = json.load(open(stageFileName))
@@ -724,23 +743,49 @@ class evolution:
                 print(sys.exc_info())
                 exit(1)
 
+            try:
+                with open('stage/editmap.pickle', 'rb') as editFitMapFile:
+                    print('[resuming GEVO] Previous EditFitMap found ... ', end='')
+                    self.editFitMap = pickle.load(editFitMapFile)
+                    print(f'{len(self.editFitMap)} entries loaded')
+            except FileNotFoundError:
+                pass
+
             popSize = len(allEdits)
-            self.mut_local_rng = [ random.Random(random.getrandbits(16)) for i in range(popSize) ]
-            self.cx_local_rng = [ random.Random(random.getrandbits(16)) for i in range(popSize) ]
-            print("Resume the population from {}. Size {}".format(stageFileName, popSize))
+
+            try:
+                print(f'[resuming GEVO] Loading random number generator from {rngFileName} ... ', end='')
+                with open(rngFileName, 'rb') as fp:
+                    rng_state = pickle.load(fp)
+                random.setstate(rng_state['global'])
+                self.rng['cx'].setstate(rng_state['cx'])
+                self.rng['mut'].setstate(rng_state['mut'])
+                self.rng['mut_local'] = [ random.Random() for i in range(popSize) ]
+                for rng, state in zip(self.rng['mut_local'], rng_state['mut_local']):
+                    rng.setstate(state)
+                self.rng['cx_local'] = [ random.Random() for i in range(popSize) ]
+                for rng, state in zip(self.rng['cx_local'], rng_state['cx_local']):
+                    rng.setstate(state)
+                print('done')
+            except FileNotFoundError:
+                print(f"GEVO Error in loading rng file \"{rngFileName}\"")
+                print(sys.exc_info())
+                exit(1)
+            
+            print(f"[resuming GEVO] Rebuilding the population from {stageFileName}. Size {popSize}")
             self.pop = self.toolbox.population(n=popSize)
             self.generation = resumeGen
 
             resultList = [False] * popSize
             for i, (edits, ind) in enumerate(zip(allEdits, self.pop)):
                 ind.update_edits(edits)
-                threadPool.append(
+                self.threadPool.append(
                     Thread(target=irind.update_from_edits, args=(i, ind, resultList))
                 )
-                threadPool[-1].start()
+                self.threadPool[-1].start()
 
             for i, ind in enumerate(self.pop):
-                threadPool[i].join()
+                self.threadPool[i].join()
                 if resultList[i] == False:
                     raise Exception(f"Could not reconstruct ind from edits:{ind.edits}")
                 fitness = self.evaluate(ind)
@@ -767,6 +812,8 @@ class evolution:
         self.mutLogFile = open('mut_stat.log', 'w')
         self.mutDistFile = open('mut_dist.csv', 'w')
         self.mutLog()
+        self.progressFile = open('progress.csv', 'w')
+        self.progressFile.write(f'{self.generation},{record["min"][0]},\n')
         minExecTime = record["min"][0]
         print("")
 
@@ -793,7 +840,7 @@ class evolution:
                 f.write(paretofGen[0][0].srcEnc.decode())
             with open("g{}_maxerr.edit".format(self.generation), 'w') as f:
                 print(paretofGen[0][0].edits, file=f)
-            with open("editmap.pickle", 'wb') as emfile:
+            with open("stage/editmap.pickle", 'wb') as emfile:
                 pickle.dump(self.editFitMap, emfile)
 
             self.generation = self.generation + 1
@@ -801,25 +848,25 @@ class evolution:
                 value['pass'].append(0)
                 value['fail'].append(0)
 
-            threadPool.clear()
+            self.threadPool.clear()
             for cnt, (child1, child2) in enumerate(zip(offspring[::2], offspring[1::2])):
                 if len(child1.edits) < 2 and len(child2.edits) < 2:
                     continue
-                if cx_rng.random() < self.CXPB:
-                    threadPool.append(
-                        Thread(target=self.toolbox.mate, args=(child1, child2, self.cx_local_rng[cnt])))
-                    threadPool[-1].start()
-            for thread in threadPool:
+                if self.rng['cx'].random() < self.CXPB:
+                    self.threadPool.append(
+                        Thread(target=self.toolbox.mate, args=(child1, child2, self.rng['cx_local'][cnt])))
+                    self.threadPool[-1].start()
+            for thread in self.threadPool:
                 thread.join()
 
-            threadPool.clear()
+            self.threadPool.clear()
             for cnt, mutant in enumerate(offspring):
-                if mut_rng.random() < self.MUPB:
+                if self.rng['mut'].random() < self.MUPB:
                     del mutant.fitness.values
-                    threadPool.append(
-                        Thread(target=self.toolbox.mutate, args=(mutant, self.mut_local_rng[cnt])))
-                    threadPool[-1].start()
-            for thread in threadPool:
+                    self.threadPool.append(
+                        Thread(target=self.toolbox.mutate, args=(mutant, self.rng['mut_local'][cnt])))
+                    self.threadPool[-1].start()
+            for thread in self.threadPool:
                 thread.join()
 
             dead_inds = [ ind for ind in self.pop if None in ind.fitness.values ]
@@ -841,6 +888,24 @@ class evolution:
                   f"e:(p:{self.evalStats['epi']['pass'][-1]},f:{self.evalStats['epi']['fail'][-1]})")
             print(self.logbook.stream)
             self.mutLog()
+            self.progressFile.write(f'{self.generation},{record["min"][0]},\n')
             self.updateSlideFromPlot()
             self.writeStage()
             print("") # an empty line as a generation separator
+
+    def stop_evolve(self):
+        sys.stderr = open(os.devnull, "w")
+        self.teminateEvent.set()
+        subprocess.run(['killall', self.appBinary])
+        subprocess.run(['killall', 'llvm-mutate'])
+        while len(self.threadPool) > 0:
+            self.threadPool = [ t for t in self.threadPool if t.is_alive() ]
+            time.sleep(0)
+
+        print("   Valid variant: {}".format(self.mutStats['valid']))
+        print(" Invalid variant: {}".format(self.mutStats['invalid']))
+        print("Infinite variant: {}".format(self.mutStats['infinite']))
+        self.mutLog()
+        if self.generation > 0:
+            self.presentation.save('progress.pptx')
+
