@@ -197,7 +197,7 @@ namespace {
 
     bool runOnModule(Module &M){
       if (deterministic)
-        gen.seed(rnd_seed);
+        gen.seed(unsigned(rnd_seed));
 
       std::string Inst1 = InstList1[0];
       InstList1.erase(InstList1.begin());
@@ -230,7 +230,7 @@ namespace {
 
     bool runOnModule(Module &M){
       if (deterministic)
-        gen.seed(rnd_seed);
+        gen.seed(unsigned(rnd_seed));
 
       Instruction *temp;
       std::string Inst1 = InstList1[0];
@@ -274,7 +274,7 @@ namespace {
 
     bool runOnModule(Module &M){
       if (deterministic)
-        gen.seed(rnd_seed);
+        gen.seed(unsigned(rnd_seed));
 
       Instruction *temp;
       std::string Inst1 = InstList1[0];
@@ -316,7 +316,7 @@ namespace {
 
     bool runOnModule(Module &M){
       if (deterministic)
-        gen.seed(rnd_seed);
+        gen.seed(unsigned(rnd_seed));
 
       int err = 0;
       std::string Inst1 = InstList1[0];
@@ -444,7 +444,7 @@ namespace {
 
     bool runOnModule(Module &M){
       if (deterministic)
-        gen.seed(rnd_seed);
+        gen.seed(unsigned(rnd_seed));
 
       std::string Inst1 = InstList1[0];
       std::string Inst2 = InstList2[0];
@@ -496,7 +496,7 @@ namespace {
 
     bool runOnModule(Module &M){
       if (deterministic)
-        gen.seed(rnd_seed);
+        gen.seed(unsigned(rnd_seed));
 
       std::string Inst1 = InstList1[0];
       std::string Inst2 = InstList2[0];
@@ -553,7 +553,7 @@ namespace {
     Cache() : ModulePass(ID) {}
     bool runOnModule(Module &M){
       if (deterministic)
-        gen.seed(rnd_seed);
+        gen.seed(unsigned(rnd_seed));
 
       std::string Inst1 = InstList1[0];
       std::string Inst2 = InstList2[0];
