@@ -138,7 +138,11 @@ if __name__ == '__main__':
             stdin=args.input_file.open(),
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE )
-        opt_stdout, opt_stderr = opt_proc.communicate()
+        try:
+            opt_stdout, opt_stderr = opt_proc.communicate()
+        except KeyboardInterrupt:
+            sys.exit(RET_ERROR)
+
         if opt_proc.returncode != 0:
             print(f"llvm-mutate: Error in {opt_proc.args}")
             print(opt_stderr.decode(), end='')
@@ -151,7 +155,11 @@ if __name__ == '__main__':
             stdin=args.input_file.open(),
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE )
-        opt_stdout, opt_stderr = opt_proc.communicate()
+        try:
+            opt_stdout, opt_stderr = opt_proc.communicate()
+        except KeyboardInterrupt:
+            sys.exit(RET_ERROR)
+
         if opt_proc.returncode != 0:
             print(f"llvm-mutate: Error in {opt_proc.args}")
             print(opt_stderr.decode(), end='')
@@ -166,7 +174,11 @@ if __name__ == '__main__':
             stdin=args.input_file.open(),
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE )
-        opt_stdout, opt_stderr = opt_proc.communicate()
+        try:
+            opt_stdout, opt_stderr = opt_proc.communicate()
+        except KeyboardInterrupt:
+            sys.exit(RET_ERROR)
+
         if opt_proc.returncode != 0:
             print(f"llvm-mutate: Error in {opt_proc.args}")
             print(opt_stderr.decode(), end='')
@@ -196,7 +208,10 @@ if __name__ == '__main__':
             stdin=subprocess.PIPE,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE )
-        opt_stdout, opt_stderr = opt_proc.communicate(input=input_str)
+        try:
+            opt_stdout, opt_stderr = opt_proc.communicate(input=input_str)
+        except KeyboardInterrupt:
+            sys.exit(RET_ERROR)
 
         if opt_proc.returncode != 0:
             print(opt_stderr.decode(), end='')
@@ -211,7 +226,11 @@ if __name__ == '__main__':
         stdin=subprocess.PIPE,
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE )
-    llvmdis_stdout, llvmdis_stderr = llvmdis_proc.communicate(input=opt_stdout)
+    try:
+        llvmdis_stdout, llvmdis_stderr = llvmdis_proc.communicate(input=opt_stdout)
+    except KeyboardInterrupt:
+        sys.exit(RET_ERROR)
+
     if llvmdis_proc.returncode != 0:
         print(f"llvm-mutate: llc error in \"{' '.join(opt_proc.args)} | {' '.join(llvmdis_proc.args)}\"")
         print(llvmdis_stderr.decode())
